//
// Copyright (c) 2022, MindFusion LLC - Bulgaria.
//

using System;
using System.ComponentModel;
using System.Data;
using System.Data.OleDb;
using System.Drawing;
using System.Windows.Forms;

using MindFusion.Diagramming.Layout;
using MindFusion.Drawing;

using ColumnStyle = MindFusion.Diagramming.ColumnStyle;


namespace MindFusion.Diagramming.WinForms.Samples.CS.DBDesign
{
	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();

			diagram.Bounds = new RectangleF(0, 0, 500, 1000);
			diagram.TableNodeStyle.Brush = new LinearGradientBrush(
                Color.FromArgb(224, 233, 233), Color.FromArgb(102, 154, 204), 30);
            diagram.TableNodeStyle.TextBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(45, 57, 86));
            diagram.TableNodeStyle.CellTextBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(45, 57, 86));
            diagram.TableNodeStyle.Stroke = new MindFusion.Drawing.SolidBrush(Color.FromArgb(192, 192, 192));
            diagram.DiagramLinkStyle.Stroke = new MindFusion.Drawing.SolidBrush(Color.FromArgb(192, 192, 192));
            diagram.DiagramLinkStyle.HeadStroke = new MindFusion.Drawing.SolidBrush(Color.FromArgb(192, 192, 192));
            diagram.DiagramLinkStyle.Brush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(0, 52, 102));
		}

		private void menuItem1_Click(object sender, EventArgs e)
		{
			Close();
		}

		private void addRow(TableNode table)
		{
			AddRowForm form = new AddRowForm();

			if (form.ShowDialog(this) == DialogResult.OK)
			{
				// this adds a row and returns the index of the new row
				int lastRow = table.AddRow();

				// use the cell indexer to access cells by their column and row
				table[0, lastRow].Text = form.FieldName;
				table[1, lastRow].Text = form.FieldType;

				// refresh SQL definition
				generateSQL();
			}
		}

		private void editRow(TableNode table, int row)
		{
			EditRowForm form = new EditRowForm();

			// get the text of cells on this row
			form.FieldName = table[0, row].Text;
			form.FieldType = table[1, row].Text;

			if (form.ShowDialog(this) == DialogResult.OK)
			{
				// set the text of cells on this row
				table[0, row].Text = form.FieldName;
				table[1, row].Text = form.FieldType;

				// refresh SQL definition
				generateSQL();
			}
		}

		private void renameTable(TableNode table)
		{
			RenameForm form = new RenameForm();
			form.TableName = table.Caption;

			if (form.ShowDialog(this) == DialogResult.OK)
			{
				table.Caption = form.TableName;
				generateSQL();
			}
		}

		private void generateSQL()
		{
			text.Text = "";

			// enumerate all tables in the current diagram
			foreach (TableNode table in diagram.Nodes)
			{
				text.Text += "CREATE TABLE " + table.Caption + "\r\n(";

				// enumerate all rows of a table
				for (int r = 0; r < table.RowCount; ++r)
				{
					// get text of cells in current row
					text.Text += "\t" + table[0, r].Text + " " + table[1, r].Text;
					if (r < table.RowCount - 1)
						text.Text += ",\r\n";
				}
				text.Text += "\r\n);\r\n\r\n";
			}
		}

		private void miTableAddRow_Click(object sender, EventArgs e)
		{
			addRow(tblClicked);
		}

		private void miTableRename_Click(object sender, EventArgs e)
		{
			renameTable(tblClicked);
		}

		private void miRowRenameTable_Click(object sender, EventArgs e)
		{
			renameTable(tblClicked);
		}

		private void miRowAdd_Click(object sender, EventArgs e)
		{
			addRow(tblClicked);
		}

		private void miRowEdit_Click(object sender, EventArgs e)
		{
			editRow(tblClicked, rowClicked);
		}

		private void miRowDelete_Click(object sender, EventArgs e)
		{
			tblClicked.DeleteRow(rowClicked);
			generateSQL();
		}

		private void miNew_Click(object sender, EventArgs e)
		{
			diagram.ClearAll();
			generateSQL();
		}

		private void miSave_Click(object sender, EventArgs e)
		{
			if (saveDlg.ShowDialog(this) == DialogResult.OK)
				diagram.SaveToFile(saveDlg.FileName, true);
		}

		private void miLoad_Click(object sender, EventArgs e)
		{
			if (openDlg.ShowDialog(this) == DialogResult.OK)
			{
				try
				{
					diagram.LoadFromFile(openDlg.FileName);
					generateSQL();
				}
				catch (Exception ex)
				{
					MessageBox.Show(ex.Message);
				}
			}
		}

		private void miCreate_Click(object sender, EventArgs e)
		{
			// create a new table with the specified extent
			TableNode tbl = diagram.Factory.CreateTableNode(
				15 + tableCount * 3, 15 + tableCount * 4, 50, 60);
			tbl.Caption = "Table" + tableCount;

			// the first column will be resized together with the table
			tbl.Columns[0].ColumnStyle = ColumnStyle.AutoWidth;

			generateSQL();
			tableCount++;
		}

		private void miRename_Click(object sender, EventArgs e)
		{
			if (diagram.ActiveItem == null)
				return;

			// check if the selected object is a table
			TableNode table = diagram.ActiveItem as TableNode;
			if (table != null)
				renameTable(table);
		}

		private void miDelete_Click(object sender, EventArgs e)
		{
			if (diagram.ActiveItem == null)
				return;

			// check if the selected object is a table
			TableNode table = diagram.ActiveItem as TableNode;
			if (table != null)
			{
				diagram.Nodes.Remove(table);
				generateSQL();
			}
		}

		private void miInfo_Click(object sender, EventArgs e)
		{
			TableNode dest = (TableNode)currentLink.Destination;
			TableNode orgn = (TableNode)currentLink.Origin;
			String sInfo = "Linking \ntable " + orgn.Caption + ", row " +
				currentLink.OriginIndex.ToString() + " to\n table" + dest.Caption +
					", row " + currentLink.DestinationIndex.ToString();
			MessageBox.Show(this, sInfo);
		}

		private void miImport_Click(object sender, EventArgs e)
		{
			ConnectForm cf = new ConnectForm();
			cf.ConnectionString =
				"Provider=SQLOLEDB;Data Source=SHTAIGATA\\VSDOTNET;Initial Catalog=Media;Integrated Security=SSPI;";

			if (cf.ShowDialog() != DialogResult.OK)
				return;

			try
			{
				OleDbConnection cn = new OleDbConnection();
				DataTable schemaTable;

				// connect using specified connection string
				cn.ConnectionString = cf.ConnectionString;
				cn.Open();

				// retrieve schema information about tables.
				// because tables include tables, views, and other objects,
				// restrict to just TABLE in the Object array of restrictions.
				schemaTable = cn.GetOleDbSchemaTable(OleDbSchemaGuid.Tables,
					new Object[] { null, null, null, "TABLE" });

				// iterate tables
				for (int i = 0; i < schemaTable.Rows.Count; i++)
				{
					TableNode tbl = new TableNode(diagram);
					tbl.Bounds = new RectangleF(0, 0, 35, 50);
					tbl.Caption = schemaTable.Rows[i].ItemArray[2].ToString();
					diagram.Nodes.Add(tbl);

					// get columns
					OleDbCommand cmd = new OleDbCommand();
					DataTable tblInfo;
					OleDbDataReader reader;

					// retrieve schema from the table into a DataReader
					cmd.Connection = cn;
					cmd.CommandText = "SELECT * FROM " + tbl.Caption;
					reader = cmd.ExecuteReader(CommandBehavior.KeyInfo);

					//Retrieve column schema into a DataTable.
					tblInfo = reader.GetSchemaTable();
					tbl.RedimTable(2, 0);

					// for each field in the table...
					int r = 0;
					foreach (DataRow field in tblInfo.Rows)
					{
						tbl.AddRow();

						// that's the name of the columns
						tbl[0, r].Text = field[0].ToString();

						// that's a .NET type, not DB one; skip 'System.'
						string dotnetType = field[5].ToString();
						tbl[1, r].Text = dotnetType.Substring(7);

						r++;
					}

					//Always close the DataReader and connection.
					reader.Close();
				}

				// explicitly close - don't wait on garbage collection.
				cn.Close();

				// TODO: read relations

				TreeLayout tl =
					new TreeLayout();
				tl.NodeDistance = 80;
				tl.Arrange(diagram);

				diagram.RouteAllLinks();
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message);
			}
		}

		private void diagram_NodeClicked(object sender, NodeEventArgs e)
		{
			if (e.MouseButton == MouseButton.Right)
			{
				// store a reference to the clicked table
				tblClicked = e.Node as TableNode;
				cmTable.Show(this, PointToClient(MousePosition));
			}
		}

		private void diagram_NodeCreated(object sender, NodeEventArgs e)
		{
			TableNode table = e.Node as TableNode;

			if (table != null)
			{
				table.Caption = "Table" + tableCount++;

				// set the first column to resize with the table
				table.Columns[0].ColumnStyle = ColumnStyle.AutoWidth;

				generateSQL();
			}
		}

		private void diagram_NodeDoubleClicked(object sender, NodeEventArgs e)
		{
			addRow(e.Node as TableNode);
		}

		private void diagram_CellDoubleClicked(object sender, CellEventArgs e)
		{
			editRow(e.Table, e.Row);

            // do not fire NodeDoubleClicked
            e.PreventNodeEvent = true;
		}

		private void diagram_CellClicked(object sender, CellEventArgs e)
		{
			if (e.MouseButton == MouseButton.Right)
			{
				// store a reference to the clicked table and row
				tblClicked = e.Table;
				rowClicked = e.Row;
				cmRow.Show(this, PointToClient(MousePosition));

                // do not fire NodeClicked
                e.PreventNodeEvent = true;
            }
		}

		private void diagram_LinkClicked(object sender, LinkEventArgs e)
		{
			if (e.MouseButton == MouseButton.Right)
			{
				currentLink = e.Link;
				cmLink.Show(this, PointToClient(MousePosition));
			}
		}


		static int tableCount = 0;

		private TableNode tblClicked = null;
		private int rowClicked = -1;
		private DiagramLink currentLink = null;
	}
}